@@ -176,7 +176,8 @@ function UIBottomPanel:draw(canvas, x, y)
 
   x, y = x + self.x, y + self.y
   self.money_font:draw(canvas, ("%7i"):format(self.ui.hospital.balance), x + 44, y + 9)
-  local month, day = self.world:getDate()
+  local game_date = self.world:date()
+  local month, day = game_date:monthOfYear(), game_date:dayOfMonth()
   self.date_font:draw(canvas, _S.date_format.daymonth:format(day, month), x + 140, y + 20, 60, 0)
 
   -- Draw possible information in the dynamic info bar
@@ -203,7 +204,7 @@ function UIBottomPanel:draw(canvas, x, y)
   self:drawReputationMeter(canvas, x + 55, y + 35)
 end
 
-function UIBottomPanel:setPosition(x, y) -- luacheck: ignore 212 keep y from parent class
+function UIBottomPanel:setPosition(x, y)
   -- Lock to bottom of screen
   return Window.setPosition(self, x, -0.1)
 end
@@ -217,10 +218,21 @@ function UIBottomPanel:drawReputationMeter(canvas, x_left, y)
   self.panel_sprites:draw(canvas, 36, x_left + math.floor(step * (self.ui.hospital.reputation - self.ui.hospital.reputation_min)), y)
 end
 
+--! Adds dynamic text to the bottom panel based on cursor position
+--!param canvas
+--!param x (num) coordinate
+--!param y (num) coordinate
 function UIBottomPanel:drawDynamicInfo(canvas, x, y)
-  if self.world:isCurrentSpeed("Pause") and not self.world.user_actions_allowed then
-    self.pause_font:drawWrapped(canvas, _S.misc.pause, x + 10, y + 14, 255, "center")
-    return
+  if self.world:isCurrentSpeed("Pause") then
+    if not self.world.user_actions_allowed then
+      -- Original pause behaviour, show pause text
+      self.pause_font:drawWrapped(canvas, _S.misc.pause, x + 10, y + 14, 255, "center")
+      return
+    elseif not (self.dynamic_info and self.dynamic_info["text"]) then
+      -- User allows editing while paused, only show pause text where dynamic text not present
+      self.pause_font:drawWrapped(canvas, _S.misc.pause, x + 10, y + 14, 255, "center")
+      return
+    end
   end
 
   if not (self.dynamic_info and self.dynamic_info["text"]) then
