@@ -271,7 +271,7 @@ bool idle_tile_finder::try_node(path_node* pNode, map_tile_flags flags,
 }
 
 bool idle_tile_finder::find_idle_tile(const level_map* pMap, int iStartX,
-                                      int iStartY, int iN) {
+                                      int iStartY, int iN, int parcelId) {
   if (pMap == nullptr) {
     pMap = parent->default_map;
   }
@@ -284,15 +284,25 @@ bool idle_tile_finder::find_idle_tile(const level_map* pMap, int iStartX,
   start_y = iStartY;
   map = pMap;
 
+  if (parcelId <= 0 || parcelId > pMap->get_parcel_count() ||
+      pMap->get_parcel_owner(parcelId) == 0)
+    parcelId = 0;
+
   path_node* pNode = init(pMap, iStartX, iStartY);
   int iWidth = pMap->get_width();
   path_node* pPossibleResult = nullptr;
 
   while (true) {
     pNode->visited = true;
-    map_tile_flags flags = pMap->get_tile_unchecked(pNode->x, pNode->y)->flags;
 
-    if (!flags.do_not_idle && flags.passable && flags.hospital) {
+    // Check whether pNode is a feasible destination.
+    const map_tile* node_tile = pMap->get_tile_unchecked(pNode->x, pNode->y);
+    map_tile_flags flags = node_tile->flags;
+
+    bool correct_parcel = parcelId == 0 || parcelId == node_tile->iParcelId;
+    if (!flags.do_not_idle && flags.passable && flags.hospital &&
+        correct_parcel) {
+      // Try to delay returning an idle tile until we find the Nth tile.
       if (iN == 0) {
         parent->destination = pNode;
         return true;
